import 'dart:io';

import 'package:audioplayers/audioplayers.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';

// 录音列表
class RecordList extends StatefulWidget {
  final List<String> records; //audio列表

  const RecordList({
    Key? key,
    required this.records,
  }) : super(key: key);

  @override
  _RecordListState createState() => _RecordListState();
}

class _RecordListState extends State<RecordList> {
  late int _totalTime; // 总时常
  late int _currentTime; // 当前播放位置
  double _percent = 0.0;
  int _selected = -1;
  bool isPlay = false;
  AudioPlayer advancedPlayer = AudioPlayer();

  @override
  Widget build(BuildContext context) {
    return ListView.builder(
      itemCount: widget.records.length,
      //滚动条
      shrinkWrap: true,
      reverse: true,
      itemBuilder: (BuildContext context, int i) {
        return Card(
          elevation: 5,
          child: ExpansionTile(
            title: Text(
              'Record ${widget.records.length - i}',
              style: TextStyle(color: Colors.black),
            ),
            subtitle: Text(
              _getTime(filePath: widget.records.elementAt(i)),
              style: TextStyle(color: Colors.black38),
            ),
            onExpansionChanged: ((newState) {
              if (newState) {
                setState(() {
                  _selected = i;
                });
              }
            }),
            children: [
              Container(
                height: 100,
                padding: const EdgeInsets.all(10),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    LinearProgressIndicator(
                      minHeight: 5,
                      backgroundColor: Colors.black,
                      valueColor: AlwaysStoppedAnimation<Color>(Colors.green),
                      value: _selected == i ? _percent : 0,
                    ),
                    Row(
                      children: [
                        (isPlay)
                            ? _Presso(
                                ico: Icons.pause,
                                onPressed: () {
                                  setState(() {
                                    isPlay = false;
                                  });
                                  advancedPlayer.pause();
                                })
                            : _Presso(
                                ico: Icons.play_arrow,
                                onPressed: () {
                                  setState(() {
                                    isPlay = true;
                                  });
                                  advancedPlayer.play(
                                      widget.records.elementAt(i),
                                      isLocal: true);
                                  setState(() {});
                                  setState(() {
                                    _selected = i;
                                    _percent = 0.0;
                                  });
                                  advancedPlayer.onPlayerCompletion.listen((_) {
                                    setState(() {
                                      _percent = 0.0;
                                    });
                                  });
                                  advancedPlayer.onDurationChanged
                                      .listen((duration) {
                                    setState(() {
                                      _totalTime = duration.inMicroseconds;
                                    });
                                  });
                                  advancedPlayer.onAudioPositionChanged
                                      .listen((duration) {
                                    setState(() {
                                      _currentTime = duration.inMicroseconds;
                                      _percent = _currentTime.toDouble() /
                                          _totalTime.toDouble();
                                    });
                                  });
                                }),
                        _Presso(
                            ico: Icons.stop,
                            onPressed: () {
                              advancedPlayer.stop();
                              setState(() {
                                _percent = 0.0;
                              });
                            }),
                        _Presso(
                            ico: Icons.delete,
                            onPressed: () {
                              Directory appDirec =
                                  Directory(widget.records.elementAt(i));
                              appDirec.delete(recursive: true);
                              Fluttertoast.showToast(msg: "File Deleted");
                              setState(() {
                                widget.records
                                    .remove(widget.records.elementAt(i));
                              });
                            }),
                        _Presso(
                            ico: Icons.share,
                            onPressed: () {
                              Directory appDirec =
                                  Directory(widget.records.elementAt(i));
                              List<String> list = List.empty(growable: true);
                              list.add(appDirec.path);
                              // Share.shareFiles(list);
                            }),
                      ],
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    ),
                  ],
                ),
              ),
            ],
          ),
        );
      },
    );
  }

  // 获取文件创建的时间（根据文件名）
  String _getTime({required String filePath}) {
    String fromPath = filePath.substring(
        filePath.lastIndexOf('/') + 1, filePath.lastIndexOf('.'));
    if (fromPath.startsWith("1", 0)) {
      DateTime dateTime =
          DateTime.fromMillisecondsSinceEpoch(int.parse(fromPath));
      int year = dateTime.year;
      int month = dateTime.month;
      int day = dateTime.day;
      int hour = dateTime.hour;
      int min = dateTime.minute;
      String dato = '$year-$month-$day--$hour:$min';
      return dato;
    } else {
      return "No Date";
    }
  }
}

/// 按钮
class _Presso extends StatelessWidget {
  final IconData ico; // 图标
  final VoidCallback onPressed; //执行事件

  const _Presso({Key? key, required this.ico, required this.onPressed})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ButtonTheme(
      minWidth: 48.0,
      child: ElevatedButton(
          child: Icon(
            ico,
            color: Colors.white,
          ),
          onPressed: onPressed),
    );
  }
}
